/*
 * Copyright (c) 2005-2010 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package fr.obeo.acceleo.gen.ui.wizards.product;

import fr.obeo.acceleo.gen.template.TemplateConstants;
import fr.obeo.acceleo.gen.template.scripts.SpecificScript;
import fr.obeo.acceleo.gen.ui.AcceleoEcoreGenUiPlugin;
import fr.obeo.acceleo.gen.ui.AcceleoGenUIMessages;
import fr.obeo.acceleo.gen.ui.wizards.product.compatibility.ProductExportJob;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductAboutHtmlWriter;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductAboutIniWriter;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductAboutPropertiesWriter;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductAcceleoIniWriter;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductActivatorWriter;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductApplicationWriter;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductBuildWriter;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductClasspathWriter;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductGenerateWriter;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductLaunchBatWriter;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductLaunchShWriter;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductMANIFESTWriter;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductPluginXMLWriter;
import fr.obeo.acceleo.gen.ui.wizards.product.writers.CreateModuleProductProjectWriter;
import fr.obeo.acceleo.tools.resources.Resources;
import fr.obeo.acceleo.tools.strings.TextSearch;

import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.osgi.service.resolver.BundleDescription;
import org.eclipse.osgi.service.resolver.State;
import org.eclipse.pde.core.plugin.IPluginModelBase;
import org.eclipse.pde.internal.core.PDECore;
import org.eclipse.pde.internal.core.exports.FeatureExportInfo;
import org.eclipse.pde.internal.core.iproduct.ILauncherInfo;
import org.eclipse.pde.internal.core.iproduct.IProduct;
import org.eclipse.pde.internal.core.iproduct.IProductModelFactory;
import org.eclipse.pde.internal.core.iproduct.IProductPlugin;
import org.eclipse.pde.internal.core.product.WorkspaceProductModel;
import org.eclipse.pde.internal.ui.PDEPluginImages;
import org.eclipse.pde.internal.ui.wizards.product.ProductFromExtensionOperation;
import org.eclipse.ui.progress.IProgressConstants;

public class CreateAcceleoProductRunnable implements IRunnableWithProgress {

	private AcceleoCreateModuleProductData productData;

	public CreateAcceleoProductRunnable(AcceleoCreateModuleProductData productData) {
		this.productData = productData;
	}

	public void run(IProgressMonitor monitor) {
		try {
			IPath destinationPath = productData.getDestinationPath();
			IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(productData.getProjectName());
			IPath location = productData.getLocationPath();
			if (!project.exists()) {
				IProjectDescription desc = project.getWorkspace().newProjectDescription(productData.getProjectName());
				if (location != null && ResourcesPlugin.getWorkspace().getRoot().getLocation().equals(location)) {
					location = null;
				}
				desc.setLocation(location);
				project.create(desc, monitor);
				project.open(monitor);
				convertProject(project, productData, monitor);
			}
			if (!project.isOpen()) {
				project.open(monitor);
			}
			IFile productFile = project.getFile(new Path(productData.getProductFileName())); //$NON-NLS-1$
			createProduct(productFile, productData, monitor);
			if (productFile.exists()) {
				StringBuffer buffer = Resources.getFileContent(productFile);
				int i = buffer.indexOf("</plugins>"); //$NON-NLS-1$
				if (i > -1) {
					boolean hasChanged1 = addPluginTextAt(buffer, i, productData.getProductID());
					boolean hasChanged2;
					if (!productData.getEclipseVersion().startsWith("3.2.")) { //$NON-NLS-1$
						hasChanged2 = addPluginTextAt(buffer, i, "org.eclipse.equinox.launcher"); //$NON-NLS-1$
					} else {
						hasChanged2 = false;
					}
					if (hasChanged1 || hasChanged2) {
						Resources.createFile(project, new Path(productData.getProductFileName()), buffer.toString(), monitor); //$NON-NLS-1$
					}
				}
				scheduleExportJob(productFile);
			}
			File targetFolder = destinationPath.toFile();
			if (targetFolder.exists()) {
				File modelFolder = destinationPath.append("model").toFile(); //$NON-NLS-1$
				if (!modelFolder.exists()) {
					modelFolder.mkdir();
				}
				File srcGenFolder = destinationPath.append("src-gen").toFile(); //$NON-NLS-1$
				if (!srcGenFolder.exists()) {
					srcGenFolder.mkdir();
				}
				CreateModuleProductLaunchBatWriter launcherBatWriter = new CreateModuleProductLaunchBatWriter();
				String launchBatText = launcherBatWriter.generate(productData);
				createFile(targetFolder, "run.bat", launchBatText); //$NON-NLS-1$
				CreateModuleProductLaunchShWriter launcherShWriter = new CreateModuleProductLaunchShWriter();
				String launcherShText = launcherShWriter.generate(productData);
				createFile(targetFolder, "run.sh", launcherShText); //$NON-NLS-1$
				List templateIDs = new ArrayList();
				List templateNames = new ArrayList();
				IFile[] templates = Resources.members(productData.getSelectedProject(), new String[] { SpecificScript.GENERATORS_EXTENSION });
				for (int i = 0; i < templates.length; i++) {
					StringBuffer templateContent = Resources.getFileContent(templates[i]);
					if (TextSearch.getRegexSearch().indexOf(templateContent.toString(),
							TemplateConstants.SCRIPT_FILE + "[ \t\r\n]*" + TemplateConstants.SCRIPT_PROPERTY_ASSIGN + "[ \t\r\n]*" + TemplateConstants.LITERAL[0]).b() > -1) { //$NON-NLS-1$  //$NON-NLS-2$
						templateIDs.add(templates[i].getFullPath().toString());
						templateNames.add(new Path(templates[i].getName()).removeFileExtension().lastSegment());
					}
				}
				CreateModuleProductAcceleoIniWriter acceleoIniWriter = new CreateModuleProductAcceleoIniWriter();
				String acceleoIniText = acceleoIniWriter.generate(new Object[] { productData, templateIDs.toArray(new String[templateIDs.size()]),
						templateNames.toArray(new String[templateNames.size()]) });
				createFile(targetFolder, "acceleo.ini", acceleoIniText); //$NON-NLS-1$
			}
		} catch (CoreException e) {
			IStatus status = new Status(IStatus.ERROR, AcceleoEcoreGenUiPlugin.getDefault().getID(), IStatus.OK, e.getMessage(), e);
			AcceleoEcoreGenUiPlugin.getDefault().getLog().log(status);
		}
	}

	protected boolean addPluginTextAt(StringBuffer buffer, int offset, String pluginID) {
		String pluginText = "<plugin id=\"" + pluginID + "\"/>"; //$NON-NLS-1$ //$NON-NLS-2$
		if (buffer.indexOf(pluginText) == -1) {
			buffer.insert(offset, "   " + pluginText + "\n   "); //$NON-NLS-1$ //$NON-NLS-2$
			return true;
		} else {
			return false;
		}
	}

	protected void createFile(File targetFolder, String fileName, String text) {
		try {
			final File launchBatFile = new File(targetFolder.getAbsolutePath() + '/' + fileName);
			final Writer writer = new BufferedWriter(new FileWriter(launchBatFile));
			writer.write(text);
			writer.close();
		} catch (final IOException e) {
			IStatus status = new Status(IStatus.ERROR, AcceleoEcoreGenUiPlugin.getDefault().getID(), IStatus.OK, e.getMessage(), e);
			AcceleoEcoreGenUiPlugin.getDefault().getLog().log(status);
		}
	}

	/**
	 * Converts the given project to an Acceleo product project.
	 * 
	 * @param project
	 *            is the project to convert
	 * @param arg
	 *            is the class used to configure all the JET generations
	 * @param monitor
	 *            is the monitor
	 */
	private void convertProject(IProject project, ProductData arg, IProgressMonitor monitor) {
		String baseFolder = "/src/" + arg.getProductID().replaceAll("\\.", "/"); //$NON-NLS-1$  //$NON-NLS-2$  //$NON-NLS-3$
		createDefaultImage(project, monitor);
		CreateModuleProductActivatorWriter activatorWriter = new CreateModuleProductActivatorWriter();
		String text = activatorWriter.generate(arg);
		IPath file = new Path(baseFolder + "/Activator.java"); //$NON-NLS-1$
		createFile(project, file, text, monitor);
		CreateModuleProductBuildWriter buildWriter = new CreateModuleProductBuildWriter();
		text = buildWriter.generate(arg);
		file = new Path("build.properties"); //$NON-NLS-1$
		createFile(project, file, text, monitor);
		CreateModuleProductClasspathWriter classpathWriter = new CreateModuleProductClasspathWriter();
		text = classpathWriter.generate(arg);
		file = new Path(".classpath"); //$NON-NLS-1$
		createFile(project, file, text, monitor);
		CreateModuleProductMANIFESTWriter manifestWriter = new CreateModuleProductMANIFESTWriter();
		text = manifestWriter.generate(arg);
		file = new Path("META-INF/MANIFEST.MF"); //$NON-NLS-1$
		createFile(project, file, text, monitor);
		CreateModuleProductProjectWriter projectWriter = new CreateModuleProductProjectWriter();
		text = projectWriter.generate(arg);
		file = new Path(".project"); //$NON-NLS-1$
		createFile(project, file, text, monitor);
		CreateModuleProductGenerateWriter generateWriter = new CreateModuleProductGenerateWriter();
		text = generateWriter.generate(arg);
		file = new Path(baseFolder + "/runner/AcceleoGenerate.java"); //$NON-NLS-1$
		createFile(project, file, text, monitor);
		CreateModuleProductPluginXMLWriter pluginXMLWriter = new CreateModuleProductPluginXMLWriter();
		text = pluginXMLWriter.generate(arg);
		file = new Path("plugin.xml"); //$NON-NLS-1$
		createFile(project, file, text, monitor);
		CreateModuleProductApplicationWriter appWriter = new CreateModuleProductApplicationWriter();
		text = appWriter.generate(arg);
		file = new Path(baseFolder + "/runner/GenRunner.java"); //$NON-NLS-1$
		createFile(project, file, text, monitor);
		CreateModuleProductAboutHtmlWriter aboutHtmlWriter = new CreateModuleProductAboutHtmlWriter();
		text = aboutHtmlWriter.generate(arg);
		file = new Path("about.html"); //$NON-NLS-1$
		createFile(project, file, text, monitor);
		CreateModuleProductAboutIniWriter aboutIniWriter = new CreateModuleProductAboutIniWriter();
		text = aboutIniWriter.generate(arg);
		file = new Path("about.ini"); //$NON-NLS-1$
		createFile(project, file, text, monitor);
		CreateModuleProductAboutPropertiesWriter aboutPropertiesWriter = new CreateModuleProductAboutPropertiesWriter();
		text = aboutPropertiesWriter.generate(arg);
		file = new Path("about.properties"); //$NON-NLS-1$
		createFile(project, file, text, monitor);
	}

	/**
	 * Creates a 'default.gif' image in the 'icons' folder of the project.
	 * 
	 * @param project
	 *            is a project
	 * @param monitor
	 *            is the monitor
	 */
	private void createDefaultImage(IProject project, IProgressMonitor monitor) {
		try {
			URL imageURL = Platform.getBundle(AcceleoEcoreGenUiPlugin.getDefault().getID()).getEntry("icons/wizard/product.gif"); //$NON-NLS-1$
			imageURL = FileLocator.toFileURL(imageURL);
			File imageFile = new File(imageURL.getFile());
			IFolder icons = project.getFolder(new Path("icons")); //$NON-NLS-1$
			if (!icons.exists()) {
				icons.create(true, true, monitor);
			}
			if (!icons.exists(new Path("default.gif"))) { //$NON-NLS-1$
				copyFile(imageFile, icons.getFile("default.gif").getLocation().toFile()); //$NON-NLS-1$
				icons.refreshLocal(1, monitor);
			}
		} catch (IOException e) {
			IStatus status = new Status(IStatus.ERROR, AcceleoEcoreGenUiPlugin.getDefault().getID(), IStatus.OK, e.getMessage(), e);
			AcceleoEcoreGenUiPlugin.getDefault().getLog().log(status);
		} catch (CoreException e) {
			IStatus status = new Status(IStatus.ERROR, AcceleoEcoreGenUiPlugin.getDefault().getID(), IStatus.OK, e.getMessage(), e);
			AcceleoEcoreGenUiPlugin.getDefault().getLog().log(status);
		}
	}

	/**
	 * Creates a file and its content.
	 * 
	 * @param project
	 *            is the project
	 * @param projectRelativePath
	 *            is the path of the file to create, relative to the project
	 * @param content
	 *            is the content of the new file
	 * @param monitor
	 *            is the monitor
	 */
	private void createFile(IProject project, IPath projectRelativePath, String content, IProgressMonitor monitor) {
		try {
			ByteArrayInputStream javaStream = new ByteArrayInputStream(content.getBytes("UTF8")); //$NON-NLS-1$
			IContainer container = project;
			String[] folders = projectRelativePath.removeLastSegments(1).segments();
			for (int i = 0; i < folders.length; i++) {
				container = container.getFolder(new Path(folders[i]));
				if (!container.exists()) {
					((IFolder) container).create(true, true, monitor);
				}
			}
			IFile file = container.getFile(new Path(projectRelativePath.lastSegment()));
			if (!file.exists()) {
				file.create(javaStream, true, monitor);
			} else {
				file.setContents(javaStream, true, false, monitor);
			}
		} catch (CoreException e) {
			IStatus status = new Status(IStatus.ERROR, AcceleoEcoreGenUiPlugin.getDefault().getID(), IStatus.OK, e.getMessage(), e);
			AcceleoEcoreGenUiPlugin.getDefault().getLog().log(status);
		} catch (UnsupportedEncodingException e) {
			IStatus status = new Status(IStatus.ERROR, AcceleoEcoreGenUiPlugin.getDefault().getID(), IStatus.OK, e.getMessage(), e);
			AcceleoEcoreGenUiPlugin.getDefault().getLog().log(status);
		}
	}

	/**
	 * Copy the given file.
	 * 
	 * @param sourceFile
	 *            is the file to copy
	 * @param targetFile
	 *            is the file to create
	 * @throws FileNotFoundException
	 *             the input file isn't found
	 * @throws IOException
	 *             if an I/O error occurs.
	 */
	private void copyFile(File sourceFile, File targetFile) throws FileNotFoundException, IOException {
		if (targetFile.exists()) {
			return;
		}
		FileInputStream in = null;
		FileOutputStream out = null;
		try {
			in = new FileInputStream(sourceFile);
			out = new FileOutputStream(targetFile);

			final int length = 8 * 1024;
			byte[] buffer = new byte[length];
			int count = 0;
			do {
				out.write(buffer, 0, count);
				count = in.read(buffer, 0, buffer.length);
			} while (count != -1);
		} finally {
			if (out != null) {
				out.close();
			}
			if (in != null) {
				in.close();
			}
		}
	}

	/**
	 * Creates a product file.
	 * 
	 * @param productFile
	 *            is the product file
	 * @param arg
	 *            is the class used to configure all the JET generations
	 * @param monitor
	 *            is the monitor
	 */
	private void createProduct(final IFile productFile, final AcceleoCreateModuleProductData arg, IProgressMonitor monitor) {
		try {
			ProductFromExtensionOperation op = new ProductFromExtensionOperation(productFile, arg.getProductID()) {
				protected void initializeProduct(IProduct product) {
					super.initializeProduct(product);
					product.setId(arg.getProductID());
					product.setApplication(arg.getApplicationID());
					product.setName("Acceleo Module Product"); //$NON-NLS-1$
					IProductModelFactory factory = product.getModel().getFactory();
					ILauncherInfo launcherInfo = factory.createLauncherInfo();
					launcherInfo.setLauncherName("acceleo"); //$NON-NLS-1$
					product.setLauncherInfo(launcherInfo);
				}

				protected void addPlugins(IProductModelFactory factory, IProduct product, IPluginModelBase[] plugins) {
					IProductPlugin[] pplugins = new IProductPlugin[plugins.length + 1 + arg.getPluginDependencies().size()];
					pplugins[0] = factory.createPlugin();
					pplugins[0].setId(productFile.getProject().getName());
					for (int i = 0; i < plugins.length; i++) {
						IProductPlugin pplugin = factory.createPlugin();
						pplugin.setId(plugins[i].getPluginBase().getId());
						pplugins[i + 1] = pplugin;
					}
					for (int i = 0; i < arg.getPluginDependencies().size(); i++) {
						IProductPlugin pplugin = factory.createPlugin();
						pplugin.setId((String) arg.getPluginDependencies().get(i));
						pplugins[plugins.length + 1 + i] = pplugin;
					}
					product.addPlugins(pplugins);
				}
			};
			productData.getWizardContainer().run(false, true, op);
		} catch (InvocationTargetException e) {
			IStatus status = new Status(IStatus.ERROR, AcceleoEcoreGenUiPlugin.getDefault().getID(), IStatus.OK, e.getMessage(), e);
			AcceleoEcoreGenUiPlugin.getDefault().getLog().log(status);
		} catch (InterruptedException e) {
			IStatus status = new Status(IStatus.ERROR, AcceleoEcoreGenUiPlugin.getDefault().getID(), IStatus.OK, e.getMessage(), e);
			AcceleoEcoreGenUiPlugin.getDefault().getLog().log(status);
		}
	}

	/**
	 * Schedules an 'export' job to be run. The job is added to a queue of
	 * waiting jobs, and will be run when it arrives at the beginning of the
	 * queue.
	 * 
	 * @param productFile
	 *            is the product file
	 */
	protected void scheduleExportJob(IFile productFile) {
		WorkspaceProductModel productModel = new WorkspaceProductModel(productFile, false);
		try {
			productModel.load();
			if (!productModel.isLoaded()) {
				MessageDialog.openError(productData.getWizardContainer().getShell(), AcceleoGenUIMessages.getString("AcceleoExportToProductWizard.ProductExportWizardError"), //$NON-NLS-1$
						AcceleoGenUIMessages.getString("AcceleoExportToProductWizard.ProductExportWizardCorrupt")); //$NON-NLS-1$
			} else {
				FeatureExportInfo info = new FeatureExportInfo();
				info.toDirectory = true;
				info.exportSource = false;
				info.destinationDirectory = productData.getDestinationPath().toString();
				info.items = getPluginModels(productModel);
				String rootDirectory = "acceleo"; //$NON-NLS-1$
				if ("".equals(rootDirectory.trim())) //$NON-NLS-1$
					rootDirectory = "."; //$NON-NLS-1$
				ProductExportJob job = new ProductExportJob(info, productModel, rootDirectory, productData.getWizardContainer());
				job.setUser(true);
				job.schedule();
				job.setProperty(IProgressConstants.ICON_PROPERTY, PDEPluginImages.DESC_FEATURE_OBJ);
			}
		} catch (CoreException e) {
			MessageDialog.openError(productData.getWizardContainer().getShell(), AcceleoGenUIMessages.getString("AcceleoExportToProductWizard.ProductExportWizardError"), //$NON-NLS-1$
					AcceleoGenUIMessages.getString("AcceleoExportToProductWizard.ProductExportWizardCorrupt")); //$NON-NLS-1$
		} catch (IllegalArgumentException e) {
			MessageDialog.openError(productData.getWizardContainer().getShell(), AcceleoGenUIMessages.getString("AcceleoExportToProductWizard.ProductExportWizardError"), //$NON-NLS-1$
					AcceleoGenUIMessages.getString("AcceleoExportToProductWizard.ProductExportWizardCorrupt")); //$NON-NLS-1$
		} catch (SecurityException e) {
			MessageDialog.openError(productData.getWizardContainer().getShell(), AcceleoGenUIMessages.getString("AcceleoExportToProductWizard.ProductExportWizardError"), //$NON-NLS-1$
					AcceleoGenUIMessages.getString("AcceleoExportToProductWizard.ProductExportWizardCorrupt")); //$NON-NLS-1$
		}
	}

	/**
	 * Get plugin dependencies.
	 * 
	 * @param productModel
	 *            is the product model (abstract representation of the product
	 *            file)
	 * @return the dependencies
	 */
	private BundleDescription[] getPluginModels(WorkspaceProductModel productModel) {
		ArrayList list = new ArrayList();
		State state = PDECore.getDefault().getModelManager().getState().getState();
		IProductPlugin[] plugins = productModel.getProduct().getPlugins();
		for (int i = 0; i < plugins.length; i++) {
			BundleDescription bundle = state.getBundle(plugins[i].getId(), null);
			if (bundle != null) {
				list.add(bundle);
				BundleDescription[] fragments = bundle.getFragments();
				for (int j = 0; j < fragments.length; j++) {
					if (!list.contains(fragments[j])) {
						list.add(fragments[j]);
					}
				}
			}
		}
		return (BundleDescription[]) list.toArray(new BundleDescription[list.size()]);
	}
}
