package fr.obeo.acceleo.gen.ui.wizards.product;

import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.wizard.IWizardContainer;
import org.osgi.framework.Bundle;

public class ProductData {

	private IProject selectedProject;

	private IWizardContainer wizardContainer;

	private IPath locationPath;

	private String projectName;

	private IPath destinationPath;

	private String productFileName;

	/**
	 * Eclipse version.
	 */
	private String eclipseVersion;

	/**
	 * New product ID.
	 */
	private String productID;

	/**
	 * New application ID.
	 */
	private String applicationID;

	/**
	 * IDs of the required plug-ins.
	 */
	private List pluginDependencies;

	/**
	 * Constructor.
	 * 
	 * @param wizard
	 *            is the wizard
	 */
	public ProductData(String productID, List pluginDependencies) {
		Bundle rt = Platform.getBundle(Platform.PI_RUNTIME);
		if (rt != null && rt.getHeaders() != null) {
			eclipseVersion = (String) rt.getHeaders().get(org.osgi.framework.Constants.BUNDLE_VERSION);
		} else {
			eclipseVersion = "3.2.0"; //$NON-NLS-1$
		}
		this.productID = productID;
		applicationID = productID + ".launch"; //$NON-NLS-1$
		this.pluginDependencies = pluginDependencies;
	}

	/**
	 * Gets the wizard container.
	 * 
	 * @return the wizard container
	 */
	public IWizardContainer getWizardContainer() {
		return wizardContainer;
	}

	/**
	 * Set the wizard container.
	 * 
	 * @param wizard
	 *            the new wizard container
	 */
	public void setWizardContainer(IWizardContainer wizardContainer) {
		this.wizardContainer = wizardContainer;
	}

	/**
	 * Gets the location path.
	 * 
	 * @return the location path
	 */
	public IPath getLocationPath() {
		return locationPath;
	}

	/**
	 * Set the location path.
	 * 
	 * @param locationPath
	 *            the new location path
	 */
	public void setLocationPath(IPath locationPath) {
		this.locationPath = locationPath;
	}

	/**
	 * Gets the project name.
	 * 
	 * @return the project name
	 */
	public String getProjectName() {
		return projectName;
	}

	/**
	 * Set the project name.
	 * 
	 * @param projectName
	 *            the new project name
	 */
	public void setProjectName(String projectName) {
		this.projectName = projectName;
	}

	/**
	 * Gets the destination path.
	 * 
	 * @return the destination path
	 */
	public IPath getDestinationPath() {
		return destinationPath;
	}

	/**
	 * Set the destination path.
	 * 
	 * @param destinationPath
	 *            the new destination path
	 */
	public void setDestinationPath(IPath destinationPath) {
		this.destinationPath = destinationPath;
	}

	/**
	 * Gets the product file name.
	 * 
	 * @return the product file name
	 */
	public String getProductFileName() {
		return productFileName;
	}

	/**
	 * Set the product file name.
	 * 
	 * @param productFileName
	 *            the new product file name
	 */
	public void setProductFileName(String productFileName) {
		this.productFileName = productFileName;
	}

	/**
	 * Gets the eclipse version.
	 * 
	 * @return the eclipse version
	 */
	public String getEclipseVersion() {
		return eclipseVersion;
	}

	/**
	 * Gets the application ID.
	 * 
	 * @return the application ID
	 */
	public String getApplicationID() {
		return applicationID;
	}

	/**
	 * Gets the product ID.
	 * 
	 * @return the product ID
	 */
	public String getProductID() {
		return productID;
	}

	/**
	 * Gets the plugin dependencies.
	 * 
	 * @return the plugin dependencies
	 */
	public List getPluginDependencies() {
		return pluginDependencies;
	}

	/**
	 * Sets the selected project.
	 * 
	 * @param selectedProject
	 *            the new selected project
	 */
	public void setSelectedProject(IProject selectedProject) {
		this.selectedProject = selectedProject;
	}

	/**
	 * Gets the selected project.
	 * 
	 * @return the selected project
	 */
	public IProject getSelectedProject() {
		return selectedProject;
	}
}
