/*
 * Copyright (c) 2005-2008 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package fr.obeo.acceleo.gen.profiler.presentation;

import fr.obeo.acceleo.gen.profiler.editor.AcceleoGenProfilerEditorMessages;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.ui.plugin.AbstractUIPlugin;

/**
 * Sort Action class for the profiler view.
 * 
 * @author Yvan LUSSAUD <a
 *         href="mailto:yvan.lussaud@obeo.fr">yvan.lussaud@obeo.fr</a>
 * 
 */
public class ProfilerSortAction extends Action {
	/** Action Image descriptor. */
	private final static ImageDescriptor sortImgDesc = AbstractUIPlugin.imageDescriptorFromPlugin("fr.obeo.acceleo.gen.profiler.editor", "icons/actions/durationSort.gif"); //$NON-NLS-1$); //$NON-NLS-2$

	/** The current sort status of this action. */
	private final ProfilerSortStatus sortStatus;

	private final TreeViewer selectionViewer;

	public ProfilerSortAction(ProfilerSortStatus sortStatus, TreeViewer selectionViewer) {
		super(AcceleoGenProfilerEditorMessages.getString("ProfilerSortAction.text"), SWT.TOGGLE); //$NON-NLS-1$
		setImageDescriptor(sortImgDesc);
		this.sortStatus = sortStatus;
		this.selectionViewer = selectionViewer;
	}

	/* (non-Javadoc) */
	public void run() {
		if (sortStatus.getSortOrder() == ProfilerSortStatus.SORT_BY_CREATION_TIME) {
			sortStatus.setSortOrder(ProfilerSortStatus.SORT_BY_TIME);
		} else {
			sortStatus.setSortOrder(ProfilerSortStatus.SORT_BY_CREATION_TIME);
		}
		selectionViewer.refresh();
	}
}
