/*
 * Copyright (c) 2005-2010 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package fr.obeo.acceleo.template.extender.api;

import fr.obeo.acceleo.gen.template.scripts.IScript;
import fr.obeo.acceleo.template.extender.internal.AcceleoExtenderCore;

/**
 * Allows to launch a Acceleo generation using extender mechanism.
 * 
 * @author Guillaume Gebhart <a
 *         href="mailto:guillaume.gebhart@obeo.fr">guillaume.gebhart@obeo.fr</a>
 * 
 */
public final class AcceleoExtender {

	/**
	 * Launching an extended Acceleo generation
	 * 
	 * @param plugin_id
	 *            The ID of the plugin that contains the extension points that
	 *            Acceleo have to use in order to extend the generation
	 * @param template
	 *            The root template
	 * @return The root Template modified
	 */
	public static IScript run(final String plugin_id, final IScript template) {
		IScript result = null;
		AcceleoExtenderCore core = AcceleoExtenderCore.getInstance();
		result = core.changeImport(plugin_id, template);

		return result == null ? template : result;
	}

	/**
	 * Launching an extended Acceleo generation
	 * 
	 * @param template
	 *            The root template
	 * @return the root template modified
	 */
	public static IScript run(final IScript template) {
		IScript result = null;
		AcceleoExtenderCore core = AcceleoExtenderCore.getInstance();
		result = core.changeImport(template);
		return result == null ? template : result;
	}

}
