/*
 * Copyright (c) 2005-2010 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package fr.obeo.acceleo.template.extender.internal;

import fr.obeo.acceleo.gen.template.TemplateSyntaxExceptions;
import fr.obeo.acceleo.gen.template.scripts.AbstractScript;
import fr.obeo.acceleo.gen.template.scripts.IScript;
import fr.obeo.acceleo.gen.template.scripts.ScriptDescriptor;
import fr.obeo.acceleo.gen.template.scripts.SpecificScript;
import fr.obeo.acceleo.template.extender.AcceleoExtenderPlugin;
import fr.obeo.acceleo.tools.plugins.AcceleoModuleProvider;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import org.eclipse.core.runtime.Path;

/**
 * Search and replace template import.
 * 
 * @author Guillaume Gebhart<a
 *         href="mailto:guillaume.gebhart@obeo.fr">guillaume.gebhart@obeo.fr</a>
 * 
 */
public final class AcceleoExtenderCore {

	private static AcceleoExtenderCore eInstance = new AcceleoExtenderCore();

	private static AcceleoExtenderHelper helper;

	/*
	 * Singleton
	 */
	private AcceleoExtenderCore() {
		helper = new AcceleoExtenderHelper();
	}

	public static AcceleoExtenderCore getInstance() {
		return eInstance;
	}

	/**
	 * Allows to change the imports of a Root Template
	 * 
	 * @param template
	 *            The root template
	 * @return AbstractScript
	 */
	public IScript changeImport(final IScript template) {
		IScript result = null;

		try {
			result = changeImports(template);
		} catch (TemplateSyntaxExceptions e) {
			AcceleoExtenderPlugin.getDefault().log(e.getMessage(), true);
		}
		return result;
	}

	/**
	 * Allows to change the imports of a Root Template
	 * 
	 * @param template
	 *            The root template
	 * @param plugin_id
	 *            The ID of the plugin that contains the extension point
	 * @return AbstractScript
	 */
	public IScript changeImport(final String plugin_id, final IScript template) {
		IScript result = null;

		try {
			result = changeImports(plugin_id, template);
		} catch (TemplateSyntaxExceptions e) {
			AcceleoExtenderPlugin.getDefault().log(e.getMessage(), true);
		}
		return result;
	}

	/**
	 * Allows to change the imports of a Root Template
	 * 
	 * @param template
	 *            The root template
	 * @param plugin_id
	 *            The ID of the plugin that contains the extension point
	 * @return The root template
	 */
	private IScript changeImports(final String plugin_id, final IScript template) throws TemplateSyntaxExceptions {
		IScript rootTemplate = null;

		Map map = helper.getTemplateExtensionsByPluginId(plugin_id);
		rootTemplate = modifyTemplate(map, template);

		return rootTemplate;
	}

	private IScript changeImports(final IScript template) throws TemplateSyntaxExceptions {
		IScript rootTemplate = null;

		Map map = helper.getAllTemplateExtensions();
		rootTemplate = modifyTemplate(map, template);

		return rootTemplate;
	}

	/**
	 * Allows to modify the Imports and scripts of the current Template
	 * 
	 * @param map
	 * @param template
	 *            The root template
	 * @return The root template
	 * @throws TemplateSyntaxExceptions
	 */
	private IScript modifyTemplate(final Map map, final IScript template) throws TemplateSyntaxExceptions {
		IScript rootTemplate = null;

		if (map != null && !map.isEmpty()) {

			Iterator it = map.keySet().iterator();
			while (it.hasNext()) {
				TemplateObject oldTemplate = (TemplateObject) it.next();
				String oldImport = oldTemplate.getPath();

				if (isTheSameTemplate(oldImport, template)) {

					String newImport = (String) map.get(oldTemplate);
					if (newTPath == null || !newImport.equals(newTPath)) { // Limit
						// the
						// acceleo
						// init
						newT = new SpecificScript(AcceleoModuleProvider.getDefault().getFile(new Path(newImport)));
						newT.reset(); // initialisation of the new Template

						newTPath = newImport;
					}

					replaceScript(newT, template);
				}

				if (template instanceof AbstractScript)
					rootTemplate = findAndReplace((AbstractScript) template, newT, oldImport);

			}
		}

		return rootTemplate;
	}

	private SpecificScript newT = null;

	private String newTPath = null;

	/**
	 * find and replace Script or Template
	 * 
	 * @param rootT
	 *            The root template
	 * @param newT
	 *            The new template
	 * @param oldImport
	 *            The template that we have to extend
	 * @param replaced
	 * @return The root Template
	 * 
	 */

	private AbstractScript findAndReplace(AbstractScript rootT, SpecificScript newT, final String oldImport) {

		ArrayList list = new ArrayList(rootT.getImports());

		ListIterator it = (ListIterator) copyImport(list);

		while (it.hasNext()) {
			Object t = it.next();

			if (t instanceof SpecificScript) {

				// int compare = ((SpecificScript)t).getFile().compareTo(oldf);
				// //can't use this because acceleo API is buged
				boolean same = isTheSameTemplate(oldImport, t);
				// //if(compare==0){
				if (same) {
					replaceScript(newT, t);
				}
				findAndReplace((SpecificScript) t, newT, oldImport);
			}
		}

		return rootT;
	}

	private void replaceScript(final SpecificScript newT, final Object t) {

		if (t instanceof SpecificScript) {
			SpecificScript currentT = (SpecificScript) t;
			Iterator it = newT.getTextTemplates().keySet().iterator();
			while (it.hasNext()) {
				Object obj = it.next();
				if (obj instanceof ScriptDescriptor) {
					ScriptDescriptor descriptor = (ScriptDescriptor) obj;
					if (currentT.getTextTemplates().containsKey(descriptor)) {
						Object value = newT.getTextTemplates().get(descriptor);
						currentT.getTextTemplates().put(descriptor, value);
					}
				}
			}
		}
	}

	private boolean isTheSameTemplate(String oldImport, Object t) {
		boolean same = false;

		File oldf = AcceleoModuleProvider.getDefault().getFile(new Path(oldImport));
		same = isTheSameTemplate(oldf, t);
		return same;
	}

	private boolean isTheSameTemplate(File oldf, Object t) {
		// use this because acceleo API is buged
		int bin_index = ((SpecificScript) t).getFile().getPath().indexOf("bin");
		bin_index = (bin_index == -1) ? 0 : bin_index;
		// f.compareTo(f2) is not valid /src => /bin
		String str = ((SpecificScript) t).getFile().getPath().substring(bin_index + "bin".length());

		return oldf.getPath().indexOf(str) > -1;
	}

	/**
	 * Allows to copy the current list.
	 * 
	 * @param x
	 *            a list
	 * @return Iterator
	 */
	private static Iterator copyImport(List x) {
		List content = new ArrayList();

		Iterator it = x.iterator();
		while (it.hasNext()) {
			Object object = it.next();
			content.add(object);
		}
		return content.listIterator();
	}

}
