/*
 * Copyright (c) 2005-2010 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package fr.obeo.acceleo.template.extender.internal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IConfigurationElement;

/***
 * Class Helper to get the extension points
 * 
 * @author Guillaume Gebhart <a
 *         href="mailto:guillaume.gebhart@obeo.fr">guillaume.gebhart@obeo.fr</a>
 * 
 */

public class AcceleoExtenderHelper {
	private static IConfigurationElement[] authorizedTemplates;

	private static Map mappings;

	static {
		authorizedTemplates = ExtensionPoint.getAllExtenderTemplates();
		mappings = getMapping(ExtensionPoint.getAllExtensionTemplates());

	}

	public Map getTemplateExtensionsByPluginId(String plugin_id) {
		return mappings;
	}

	public Map getAllTemplateExtensions() {
		return mappings;
	}

	private static Map getMapping(final IConfigurationElement[] extensions) {
		final Map map = new HashMap();
		for (int i = 0; i < extensions.length; i++) {
			String newTemplatePath = ExtensionPoint.getPath(extensions[i]);
			List result = getOldTemplates(extensions[i]);
			Iterator it = result.iterator();
			while (it.hasNext()) {
				map.put(it.next(), newTemplatePath);
			}

		}
		return map;
	}

	private static List getOldTemplates(IConfigurationElement newT) {
		final List result = new ArrayList();
		IConfigurationElement[] oldTemplates = newT.getChildren(ExtensionPoint.ATT_OLD);
		for (int i = 0; i < oldTemplates.length; i++) {
			String path = ExtensionPoint.getTemplate(oldTemplates[i]);
			if (replacementAuthorized(path))
				result.add(new TemplateObject(path));
		}
		return result;
	}

	private static boolean replacementAuthorized(final String template) {
		int size = authorizedTemplates.length;

		for (int i = 0; i < size; i++) {
			String path = ExtensionPoint.getPath(authorizedTemplates[i]);
			if (path != null && path.equals(template))
				return true;
		}
		return false;
	}

}
