/*
 * Copyright (c) 2005-2010 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package fr.obeo.acceleo.template.extender.internal;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.Platform;

/**
 * Allows to get Extension points.
 * 
 * @author Guillaume Gebhart<a
 *         href="mailto:guillaume.gebhart@obeo.fr">guillaume.gebhart@obeo.fr</a>
 * 
 */
public class ExtensionPoint {

	/**
	 * all extension points Attributes
	 */

	public static final String ATT_ID = "Id";

	public static final String ATT_PATH = "Path";

	public static final String ATT_NAME = "Name";

	public static final String ATT_OLD = "old";

	public static final String ATT_NEW = "new";

	public static final String ATT_TEMPLATE = "Template";

	public static final String ATT_DESCRIPTION = "Description";

	private static final String EXTENDER_TEMPLATE_ID = "fr.obeo.acceleo.template.extender";

	private static final String EXTENSION_TEMPLATE_ID = "fr.obeo.acceleo.template.extension";

	/**
	 * Method that allows to get all templates extender contributing the
	 * fr.obeo.acceleo.template.extender extension point.
	 * 
	 * @return all templates extender registered
	 */
	public static IConfigurationElement[] getAllExtenderTemplates() {

		return getExtensionElt(EXTENDER_TEMPLATE_ID);
	}

	/**
	 * Method that allows to get all templates extensions contributing the
	 * fr.obeo.acceleo.template.extension extension point.
	 * 
	 * @return all templates registered
	 */
	public static IConfigurationElement[] getAllExtensionTemplates() {

		return getExtensionElt(EXTENSION_TEMPLATE_ID);

	}

	/**
	 * get extension point
	 * 
	 * @param id
	 *            the ID of the extension point
	 * @return IConfigurationElement[]
	 */
	private static IConfigurationElement[] getExtensionElt(String id) {
		final IExtensionPoint point = Platform.getExtensionRegistry().getExtensionPoint(id);
		final IExtension[] extensions = point.getExtensions();
		final ArrayList configElements = new ArrayList();
		final int size = extensions.length;

		for (int i = 0; i < size; i++) {
			configElements.addAll(Arrays.asList(extensions[i].getConfigurationElements()));
		}

		return (IConfigurationElement[]) configElements.toArray(new IConfigurationElement[] {});
	}

	/**
	 * get All template extension points
	 * 
	 * @param plugin_id
	 *            the plugin's id
	 * @return All the template extension points
	 */
	public static IConfigurationElement[] getAllExtensionTemplateFilteredByPluginId(String plugin_id) {
		final IConfigurationElement[] elements = getAllExtensionTemplates();
		final List result = new ArrayList();
		for (int i = 0; i < elements.length; i++) {
			String name = elements[i].getContributor().getName();
			if (name.equals(plugin_id))
				result.add(elements[i]);
		}

		return (IConfigurationElement[]) result.toArray(new IConfigurationElement[] {});

	}

	/**
	 * Method that allows to get a template contributing the
	 * fr.obeo.acceleo.template.extension extension point from its id.
	 * 
	 * @param name
	 *            the template's id
	 * 
	 * @return the template element
	 */
	public static IConfigurationElement getExtensionTemplateConfigElement(String id) {
		IConfigurationElement configuration = null;

		IConfigurationElement[] configElements = getExtensionElt(EXTENSION_TEMPLATE_ID);
		if (configElements != null) {

			for (int i = 0; i < configElements.length; i++) {
				if (configElements[i].getAttribute(ATT_ID).equals(id))
					configuration = configElements[i];
			}
		}
		return configuration;
	}

	/*
	 * 
	 * FACILITIES
	 */

	public static String getPath(IConfigurationElement config) {

		return config.getAttribute(ATT_PATH);
	}

	public static String getTemplate(IConfigurationElement config) {

		return config.getAttribute(ATT_TEMPLATE);
	}

}
