/*
 * Copyright (c) 2005-2010 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package fr.obeo.acceleo.template.extender.services;

import fr.obeo.acceleo.gen.template.Template;
import fr.obeo.acceleo.gen.template.TemplateElement;
import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.LaunchManager;
import fr.obeo.acceleo.gen.template.scripts.IExternalJavaService;
import fr.obeo.acceleo.gen.template.scripts.IScript;
import fr.obeo.acceleo.gen.template.scripts.SpecificScript;
import fr.obeo.acceleo.template.extender.AcceleoExtenderPlugin;

import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.emf.ecore.EObject;

/**
 * Extender Services
 * 
 * @author Guillaume Gebhart<a
 *         href="mailto:guillaume.gebhart@obeo.fr">guillaume.gebhart@obeo.fr</a>
 * 
 */
public class AcceleoExtenderServices implements IExternalJavaService {

	/**
	 * Allows to call its Super Script.
	 * 
	 * @param node
	 * @return String
	 */
	public ENode nSuper(ENode node) {
		ENode result = null;
		try {
			EObject eObject = node.getEObject();
			TemplateElement elt = node.getContainerTemplateElement();
			String templateName = ((Template) elt.getParent()).getDescriptor().getName();
			SpecificScript sc = ((SpecificScript) elt.getScript());
			Set imports = sc.getImports();
			Iterator it = imports.iterator();
			while (it.hasNext()) {
				Object imp = it.next();
				if (imp instanceof IScript) {
					Template t = ((IScript) imp).getTextTemplateForEObject(eObject, templateName);
					LaunchManager manager = LaunchManager.create("run", false);

					if (t != null) {
						result = t.evaluate(eObject, manager);

					}
				}
			}
		} catch (Exception e) {
			AcceleoExtenderPlugin.getDefault().log(e.getMessage(), true);
		}
		return result;
	}

	/**
	 * Return the deprecated methods.
	 */
	public Method[] getDeprecatedMethods() {
		return null;
	}

}
